import { DefaultParamsProviderPort } from '@modules/geography/core/application/ports/default-params-provider.port';
import {
  Path,
  PathType,
  Route,
} from '@modules/geography/core/domain/route.types';
import { PARAMS_PROVIDER } from '@modules/geography/geography.di-tokens';
import { GraphhopperGeorouter } from '@modules/geography/infrastructure/graphhopper-georouter';
import { HttpService } from '@nestjs/axios';
import { Test, TestingModule } from '@nestjs/testing';

const driverPath: Path = {
  type: PathType.DRIVER,
  points: [
    {
      lon: 6,
      lat: 47,
    },
    {
      lon: 6.1,
      lat: 47.1,
    },
    {
      lon: 6.2,
      lat: 47.2,
    },
  ],
};

const mockHttpService = {};

const mockDefaultParamsProvider: DefaultParamsProviderPort = {
  getParams: jest.fn().mockImplementation(() => ({
    GEOROUTER_URL: 'http://localhost:8989',
  })),
};

describe('Graphhopper Georouter', () => {
  let graphhopperGeorouter: GraphhopperGeorouter;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [],
      providers: [
        GraphhopperGeorouter,
        {
          provide: HttpService,
          useValue: mockHttpService,
        },
        {
          provide: PARAMS_PROVIDER,
          useValue: mockDefaultParamsProvider,
        },
      ],
    }).compile();

    graphhopperGeorouter =
      module.get<GraphhopperGeorouter>(GraphhopperGeorouter);
  });

  it('should be defined', () => {
    expect(graphhopperGeorouter).toBeDefined();
  });

  it('should return basic driver routes', async () => {
    const paths: Path[] = [driverPath];
    const driverRoutes: Route[] = await graphhopperGeorouter.routes(paths, {
      detailedDistance: false,
      detailedDuration: false,
      points: true,
    });
    expect(driverRoutes.length).toBe(1);
  });
});
